"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.SitePages = void 0;
var web_1 = require("./web");
var utils_1 = require("../utils");
var contextInfo_1 = require("./contextInfo");
/**
 * Site Pages
 */
exports.SitePages = (function (url, targetInfo) {
    var sitePages = new utils_1.Base(targetInfo);
    // Default the properties
    sitePages.targetInfo.defaultToWebFl = true;
    sitePages.targetInfo.endpoint = "SitePages";
    // See if the web url exists
    if (url) {
        // Set the settings
        sitePages.targetInfo.url = url;
    }
    // Add the methods
    utils_1.Request.addMethods(sitePages, { __metadata: { type: "SP.Publishing.SitePageService" } });
    // Return the site pages
    return sitePages;
});
// Static method to convert a modern page type
exports.SitePages.convertPage = function (pageUrl, layout, webUrl) {
    // Return a promise
    return new Promise(function (resolve, reject) {
        // Get the page
        var getPage = function (pageUrl) {
            // Return a promise
            return new Promise(function (resolve, reject) {
                // See if the web url is specified
                if (webUrl) {
                    // Get the context info
                    contextInfo_1.ContextInfo.getWeb(webUrl).execute(function (context) {
                        // Get the page
                        web_1.Web(webUrl, {
                            requestDigest: context.GetContextWebInformation.FormDigestValue
                        }).Lists("Site Pages").Items().query({
                            Filter: "FileLeafRef eq '" + pageUrl + "'"
                        }).execute(function (items) {
                            // Resolve the request
                            resolve(items.results[0]);
                        }, reject);
                    }, reject);
                }
                else {
                    // Get the page
                    web_1.Web().Lists("Site Pages").Items().query({
                        Filter: "FileLeafRef eq '" + pageUrl + "'"
                    }).execute(function (items) {
                        // Resolve the request
                        resolve(items.results[0]);
                    }, reject);
                }
            });
        };
        // Get the page
        getPage(pageUrl).then(function (item) {
            // Update the item
            item.update({ PageLayoutType: layout }).execute(resolve, reject);
        }, function () {
            // Log
            console.error("Unable to get the page: " + pageUrl);
            // Reject the request
            reject();
        });
    });
};
// Static method to create a modern page
exports.SitePages.createPage = (function (pageName, pageTitle, pageTemplate, url, targetInfo) {
    var getContext = function () {
        return new Promise(function (resolve) {
            // See if the web url doesn't exists
            if (url == null) {
                resolve(null);
            }
            // Get the context of target web
            contextInfo_1.ContextInfo.getWeb(url).execute(function (context) {
                // Resolve the request
                resolve(context.GetContextWebInformation.FormDigestValue);
            });
        });
    };
    // Return a promise
    return new Promise(function (resolve, reject) {
        // Method called after the updates have completed
        var onComplete = function (itemId, fileUrl) {
            var web = web_1.Web(url, targetInfo);
            var results = { file: null, item: null, page: null };
            // Get the file
            web.getFileByUrl(fileUrl).query({ Select: ["*", "ListId"] }).execute(function (file) {
                // Set the file
                results.file = file;
                // Get the list
                web.Lists().getById(file.ListId).Items(itemId).execute(function (item) {
                    // Set the item
                    results.item = item;
                    // Get the page
                    exports.SitePages(url, targetInfo).Pages(itemId).execute(function (page) {
                        // Set the page
                        results.page = page;
                        // Resolve the request
                        resolve(results);
                    }, reject);
                }, reject);
            }, reject);
        };
        // Get the context information
        getContext().then(function (requestDigest) {
            // Set the target information
            targetInfo = __assign({ requestDigest: requestDigest }, targetInfo);
            // Create the page
            exports.SitePages(url, targetInfo).Pages().createAppPage({
                Title: pageTitle,
                PageLayoutType: pageTemplate
            }).execute(function (page) {
                // Update the file name
                web_1.Web(url, targetInfo).Lists("Site Pages").Items(page.Id).update({
                    FileLeafRef: pageName
                }).execute(
                // Updated the file name successfully
                function () {
                    // Update the file url
                    var idx = page.Url.lastIndexOf('/');
                    var fileUrl = page.Url.substring(0, idx + 1) + pageName;
                    // Complete the request
                    onComplete(page.Id, fileUrl);
                }, 
                // Unable to update the file name, but still return the object
                function () {
                    // Complete the request
                    onComplete(page.Id, page.Url);
                });
            }, reject);
        });
    });
});
